import nbformat as nbf, os, shutil, ntpath
from datetime import date
from getpass import getuser


def create_fab_notebook(name, rootdir, from_template):
    # If creating from scratch
    if not from_template:
        # Create new notebook
        nb = nbf.v4.new_notebook()
        # Initialize markdown cells
        text = [
            f"""\
<img src="./include/img/fabric_logo.png" width="500"/>

# {name}
```
Created: {date.today()}
Last Modified: 
{getuser()}
Clemson University
<username>@clemson.edu
```

<div style="page-break-after: always; visibility: hidden"> 
\pagebreak 
</div>""",
        ]
        # Initialize code cells
        code = [
            """\
from fabrictestbed_extensions.fablib.fablib import FablibManager as fablib_manager

try:
    fablib = fablib_manager()
                    
    fablib.show_config()
except Exception as e:
    print(f"Exception: {e}")""",
            """\
# Clean up log directory to avoid large files
import os
for item in os.scandir(os.path.join(os.getcwd(), 'logs')):
    if '.' != item.name[0]:
        os.remove(item.path)""",
        ]
        # Add cells to notebook object
        nb["cells"] = [
            nbf.v4.new_markdown_cell(text[0]),
            nbf.v4.new_code_cell(code[0]),
            nbf.v4.new_code_cell(code[1]),
        ]
        # Create notebook in new project dir
        nbf.write(nb, rootdir + f"/{name.lower()}.ipynb")
        # Skip next part
        return
    # dirname of pip package location's ipycreate.py
    dirname = os.path.dirname(__file__)
    template_path = os.path.join(dirname, f"templates/{from_template}")
    # If from_template specified
    if not os.path.exists(template_path):
        # Check if user entered a full path
        if os.path.exists(from_template):
            from_template = ntpath.basename(from_template)
            shutil.copy(from_template, os.path.join(rootdir, from_template))
        # Check if user entered local path
        elif os.path.exists(path := os.path.join(os.getcwd(), from_template)):
            shutil.copy(path, os.path.join(rootdir, from_template))
        else:
            print("Error: Incorrect usage of -f. Make sure template name is valid.")
        return

    # Iterate through items in case subfolders/more than one file exists
    for item in os.scandir(template_path):
        if os.path.isdir(item.path):
            shutil.copytree(
                item.path, os.path.join(rootdir, item.name), dirs_exist_ok=True
            )
        else:
            shutil.copy(item.path, os.path.join(rootdir, item.name))



# fabexp.py
import click, os, sys, shutil, ntpath

# get path of package file
dirname = os.path.dirname(__file__)
sys.path.append(dirname)
import modules.ipyCreate as ipyCreate

# get dir of where script is being run (user side)
cwd = os.getcwd()


@click.command()
@click.option("--add", "-a", is_flag=True, help="Creates a new project")
@click.option(
    "--name",
    "-n",
    default="myFabricExperiment",
    show_default=True,
    help="Name of your fabric project",
)
@click.option(
    "--local",
    "-l",
    is_flag=True,
    show_default=True,
    help="Include folder to store local machine scripts",
)
@click.option(
    "--from-template",
    "-f",
    default=None,
    show_default=True,
    help="Template to base your project on",
)


# main program - runs when script is called
def create_project(add, name, local, from_template):
    """
    This tool will help you create new fabric project templates, including
    directory structure. Can also specify to build off a template
    """
    # If -a flag passed
    if add:
        confirmation = input(
            f"Do you want to add a new project titled {name} in the current directory? (yes/no) "
        )
        if confirmation == "yes":
            # Get root directory of PROJECT called name
            rootdir = os.path.join(cwd, name)

            # Check to see if user entered anything invalid
            template_path = os.path.join(
                dirname, "modules/", f"templates/{from_template}"
            )
            if not os.path.exists(template_path):
                # Check if user entered a full path
                if not (
                    os.path.exists(from_template)
                    or os.path.exists(os.path.join(os.getcwd(), from_template))
                ):
                    print(
                        "Error: Incorrect usage of -f. Make sure template name is valid."
                    )
                    exit()
            # Create directories - these are default
            os.makedirs(rootdir)
            os.makedirs(os.path.join(rootdir, "upload"))
            os.makedirs(os.path.join(rootdir, "download"))
            os.makedirs(os.path.join(rootdir, "include/img"))
            os.makedirs(os.path.join(rootdir, "logs"))
            # Only create if user wants to run local scripts
            if local:
                os.makedirs(os.path.join(rootdir, "local"))
            # Copy the fabric logo from the package to the new project
            shutil.copy(
                os.path.join(dirname, "modules/img/fabric_logo.png"),
                os.path.join(rootdir, "include/img/fabric_logo.png"),
            )
            # Create the experiment notebook (either from template or from scratch)
            ipyCreate.create_fab_notebook(name, rootdir, from_template)


# Entrypoint
if __name__ == "__main__":
    create_project()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['fabexp_helper', 'fabexp_helper.modules']

package_data = \
{'': ['*'],
 'fabexp_helper.modules': ['img/*',
                           'templates/accessing_ipv4_services_from_ipv6_nodes/*',
                           'templates/accessing_ipv4_services_from_ipv6_nodes/upload/*',
                           'templates/basic_gpu_devices/*',
                           'templates/basic_nvme_devices/*',
                           'templates/benchmarking_storage/*',
                           'templates/create_l2network_basic/*',
                           'templates/create_l2network_wide_area/*',
                           'templates/create_l3network_fabnet_ipv4/*',
                           'templates/create_l3network_fabnet_ipv6/*',
                           'templates/create_slice/*',
                           'templates/customizing_nodes/*',
                           'templates/delete_slice/*',
                           'templates/fablib_common/*',
                           'templates/facility_port/*',
                           'templates/get_nodes/*',
                           'templates/hello_fabric/*',
                           'templates/hello_fabric/include/figs/*',
                           'templates/parallel_config/*',
                           'templates/persistent_storage/*',
                           'templates/renew_slice/*',
                           'templates/save_and_load/*',
                           'templates/sites_and_resources/*',
                           'templates/slices/*',
                           'templates/ssh_to_nodes/*',
                           'templates/upload_and_execute/*',
                           'templates/upload_and_execute/upload/*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'nbformat>=5.7.3,<6.0.0']

entry_points = \
{'console_scripts': ['fabexp = fabexp_helper.fabexp:create_project']}

setup_kwargs = {
    'name': 'fabexp-helper',
    'version': '0.0.2',
    'description': 'A tool to help create and manage fabric experiments',
    'long_description': '# fabexp-helper\nA CLI to help create new fabric project templates in an effort to standardize experiment structure.\n\n## Usage\n`fabexp --add (-a) --name (-n) <fabric project name> --local (-l) --from-template (-f) TEMPLATE_NAME`\n- Replace <fabric project name> with a desired name\n- `--add`: required to add project\n- `--name`: optional, default: myFabricExperiment\n- `--local`: add folder for executing local scripts, default: False\n- `--from_template`: create project from template. Names are any valid jupyter example name.\n    * Find project name from folder of jupyter examples OR your own notebook\n    * Must give path to file (relative or absolute) if custom\n    * Examples: hello_fabric, create_l2network_basic (jupyter examples)\n    * Only including examples from **fablib_api** folder for now.\nOnce you run the above command, you should see a new set of folders that serve as a template\nfor your fabric project.',
    'author': 'Devin Lane',
    'author_email': 'silvertenor@icloud.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
